@@ -26,6 +26,8 @@
 #ifndef	_LIBZUTIL_H
 #define	_LIBZUTIL_H extern __attribute__((visibility("default")))
 
+#include <string.h>
+#include <locale.h>
 #include <sys/nvpair.h>
 #include <sys/fs/zfs.h>
 
@@ -267,6 +269,18 @@ int for_each_vdev_in_nvlist(nvlist_t *nvroot, pool_vdev_iter_f func,
 void update_vdevs_config_dev_sysfs_path(nvlist_t *config);
 _LIBZUTIL_H void update_vdev_config_dev_sysfs_path(nvlist_t *nv,
     const char *path, const char *key);
+
+/*
+ * Thread-safe strerror() for use in ZFS libraries
+ */
+static inline char *zfs_strerror(int errnum) {
+#ifdef HAVE_STRERROR_L
+	return (strerror_l(errnum, uselocale(0)));
+#else
+	return (strerror(errnum));
+#endif
+}
+
 #ifdef	__cplusplus
 }
 #endif
