@@ -35,79 +35,44 @@
 #include <linux/major.h>
 #include <linux/msdos_fs.h>	/* for SECTOR_* */
 #include <linux/bio.h>
-
-#ifdef HAVE_BLK_MQ
 #include <linux/blk-mq.h>
-#endif
-
-#ifndef HAVE_BLK_QUEUE_FLAG_SET
-static inline void
-blk_queue_flag_set(unsigned int flag, struct request_queue *q)
-{
-	queue_flag_set(flag, q);
-}
-#endif
-
-#ifndef HAVE_BLK_QUEUE_FLAG_CLEAR
-static inline void
-blk_queue_flag_clear(unsigned int flag, struct request_queue *q)
-{
-	queue_flag_clear(flag, q);
-}
-#endif
 
 /*
- * 4.7 API,
- * The blk_queue_write_cache() interface has replaced blk_queue_flush()
- * interface.  However, the new interface is GPL-only thus we implement
- * our own trivial wrapper when the GPL-only version is detected.
- *
- * 2.6.36 - 4.6 API,
- * The blk_queue_flush() interface has replaced blk_queue_ordered()
- * interface.  However, while the old interface was available to all the
- * new one is GPL-only.   Thus if the GPL-only version is detected we
- * implement our own trivial helper.
+ * 6.11 API
+ * Setting the flush flags directly is no longer possible; flush flags are set
+ * on the queue_limits structure and passed to blk_disk_alloc(). In this case
+ * we remove this function entirely.
  */
+#if !defined(HAVE_BLK_ALLOC_DISK_2ARG) || \
+	!defined(HAVE_BLKDEV_QUEUE_LIMITS_FEATURES)
 static inline void
-blk_queue_set_write_cache(struct request_queue *q, bool wc, bool fua)
+blk_queue_set_write_cache(struct request_queue *q, bool on)
 {
-#if defined(HAVE_BLK_QUEUE_WRITE_CACHE_GPL_ONLY)
-	if (wc)
+	if (on) {
 		blk_queue_flag_set(QUEUE_FLAG_WC, q);
-	else
-		blk_queue_flag_clear(QUEUE_FLAG_WC, q);
-	if (fua)
 		blk_queue_flag_set(QUEUE_FLAG_FUA, q);
-	else
+	} else {
+		blk_queue_flag_clear(QUEUE_FLAG_WC, q);
 		blk_queue_flag_clear(QUEUE_FLAG_FUA, q);
-#elif defined(HAVE_BLK_QUEUE_WRITE_CACHE)
-	blk_queue_write_cache(q, wc, fua);
-#elif defined(HAVE_BLK_QUEUE_FLUSH_GPL_ONLY)
-	if (wc)
-		q->flush_flags |= REQ_FLUSH;
-	if (fua)
-		q->flush_flags |= REQ_FUA;
-#elif defined(HAVE_BLK_QUEUE_FLUSH)
-	blk_queue_flush(q, (wc ? REQ_FLUSH : 0) | (fua ? REQ_FUA : 0));
-#else
-#error "Unsupported kernel"
-#endif
+	}
 }
+#endif /* !HAVE_BLK_ALLOC_DISK_2ARG || !HAVE_BLKDEV_QUEUE_LIMITS_FEATURES */
 
 static inline void
 blk_queue_set_read_ahead(struct request_queue *q, unsigned long ra_pages)
 {
 #if !defined(HAVE_BLK_QUEUE_UPDATE_READAHEAD) && \
 	!defined(HAVE_DISK_UPDATE_READAHEAD)
-#ifdef HAVE_BLK_QUEUE_BDI_DYNAMIC
+#if defined(HAVE_BLK_QUEUE_BDI_DYNAMIC)
 	q->backing_dev_info->ra_pages = ra_pages;
+#elif defined(HAVE_BLK_QUEUE_DISK_BDI)
+	q->disk->bdi->ra_pages = ra_pages;
 #else
 	q->backing_dev_info.ra_pages = ra_pages;
 #endif
 #endif
 }
 
-#ifdef HAVE_BIO_BVEC_ITER
 #define	BIO_BI_SECTOR(bio)	(bio)->bi_iter.bi_sector
 #define	BIO_BI_SIZE(bio)	(bio)->bi_iter.bi_size
 #define	BIO_BI_IDX(bio)		(bio)->bi_iter.bi_idx
@@ -115,15 +80,6 @@ blk_queue_set_read_ahead(struct request_queue *q, unsigned long ra_pages)
 #define	bio_for_each_segment4(bv, bvp, b, i)	\
 	bio_for_each_segment((bv), (b), (i))
 typedef struct bvec_iter bvec_iterator_t;
-#else
-#define	BIO_BI_SECTOR(bio)	(bio)->bi_sector
-#define	BIO_BI_SIZE(bio)	(bio)->bi_size
-#define	BIO_BI_IDX(bio)		(bio)->bi_idx
-#define	BIO_BI_SKIP(bio)	(0)
-#define	bio_for_each_segment4(bv, bvp, b, i)	\
-	bio_for_each_segment((bvp), (b), (i))
-typedef int bvec_iterator_t;
-#endif
 
 static inline void
 bio_set_flags_failfast(struct block_device *bdev, int *flags, bool dev,
@@ -164,7 +120,6 @@ bio_set_flags_failfast(struct block_device *bdev, int *flags, bool dev,
 #define	DISK_NAME_LEN	32
 #endif /* DISK_NAME_LEN */
 
-#ifdef HAVE_BIO_BI_STATUS
 static inline int
 bi_status_to_errno(blk_status_t status)
 {
@@ -238,42 +193,6 @@ errno_to_bi_status(int error)
 		return (BLK_STS_IOERR);
 	}
 }
-#endif /* HAVE_BIO_BI_STATUS */
-
-/*
- * 4.3 API change
- * The bio_endio() prototype changed slightly.  These are helper
- * macro's to ensure the prototype and invocation are handled.
- */
-#ifdef HAVE_1ARG_BIO_END_IO_T
-#ifdef HAVE_BIO_BI_STATUS
-#define	BIO_END_IO_ERROR(bio)		bi_status_to_errno(bio->bi_status)
-#define	BIO_END_IO_PROTO(fn, x, z)	static void fn(struct bio *x)
-#define	BIO_END_IO(bio, error)		bio_set_bi_status(bio, error)
-static inline void
-bio_set_bi_status(struct bio *bio, int error)
-{
-	ASSERT3S(error, <=, 0);
-	bio->bi_status = errno_to_bi_status(-error);
-	bio_endio(bio);
-}
-#else
-#define	BIO_END_IO_ERROR(bio)		(-(bio->bi_error))
-#define	BIO_END_IO_PROTO(fn, x, z)	static void fn(struct bio *x)
-#define	BIO_END_IO(bio, error)		bio_set_bi_error(bio, error)
-static inline void
-bio_set_bi_error(struct bio *bio, int error)
-{
-	ASSERT3S(error, <=, 0);
-	bio->bi_error = error;
-	bio_endio(bio);
-}
-#endif /* HAVE_BIO_BI_STATUS */
-
-#else
-#define	BIO_END_IO_PROTO(fn, x, z)	static void fn(struct bio *x, int z)
-#define	BIO_END_IO(bio, error)		bio_endio(bio, error);
-#endif /* HAVE_1ARG_BIO_END_IO_T */
 
 /*
  * 5.15 MACRO,
@@ -301,6 +220,19 @@ zfs_check_disk_status(struct block_device *bdev)
 #endif
 }
 
+/*
+ * 5.17 API change
+ *
+ * GENHD_FL_EXT_DEVT flag removed
+ * GENHD_FL_NO_PART_SCAN renamed GENHD_FL_NO_PART
+ */
+#ifndef HAVE_GENHD_FL_EXT_DEVT
+#define	GENHD_FL_EXT_DEVT	(0)
+#endif
+#ifndef HAVE_GENHD_FL_NO_PART
+#define	GENHD_FL_NO_PART	(GENHD_FL_NO_PART_SCAN)
+#endif
+
 /*
  * 4.1 API,
  * 3.10.0 CentOS 7.x API,
@@ -363,9 +295,6 @@ zfs_check_media_change(struct block_device *bdev)
  * The function was exported for use, prior to this it existed but the
  * symbol was not exported.
  *
- * 4.4.0-6.21 API change for Ubuntu
- * lookup_bdev() gained a second argument, FMODE_*, to check inode permissions.
- *
  * 5.11 API change
  * Changed to take a dev_t argument which is set on success and return a
  * non-zero error code on failure.
@@ -383,15 +312,6 @@ vdev_lookup_bdev(const char *path, dev_t *dev)
 	*dev = bdev->bd_dev;
 	bdput(bdev);
 
-	return (0);
-#elif defined(HAVE_MODE_LOOKUP_BDEV)
-	struct block_device *bdev = lookup_bdev(path, FMODE_READ);
-	if (IS_ERR(bdev))
-		return (PTR_ERR(bdev));
-
-	*dev = bdev->bd_dev;
-	bdput(bdev);
-
 	return (0);
 #else
 #error "Unsupported kernel"
@@ -411,56 +331,24 @@ vdev_lookup_bdev(const char *path, dev_t *dev)
 static inline void
 bio_set_op_attrs(struct bio *bio, unsigned rw, unsigned flags)
 {
-#if defined(HAVE_BIO_BI_OPF)
 	bio->bi_opf = rw | flags;
-#else
-	bio->bi_rw |= rw | flags;
-#endif /* HAVE_BIO_BI_OPF */
 }
 #endif
 
 /*
  * bio_set_flush - Set the appropriate flags in a bio to guarantee
  * data are on non-volatile media on completion.
- *
- * 2.6.37 - 4.8 API,
- *   Introduce WRITE_FLUSH, WRITE_FUA, and WRITE_FLUSH_FUA flags as a
- *   replacement for WRITE_BARRIER to allow expressing richer semantics
- *   to the block layer.  It's up to the block layer to implement the
- *   semantics correctly. Use the WRITE_FLUSH_FUA flag combination.
- *
- * 4.8 - 4.9 API,
- *   REQ_FLUSH was renamed to REQ_PREFLUSH.  For consistency with previous
- *   OpenZFS releases, prefer the WRITE_FLUSH_FUA flag set if it's available.
- *
- * 4.10 API,
- *   The read/write flags and their modifiers, including WRITE_FLUSH,
- *   WRITE_FUA and WRITE_FLUSH_FUA were removed from fs.h in
- *   torvalds/linux@70fd7614 and replaced by direct flag modification
- *   of the REQ_ flags in bio->bi_opf.  Use REQ_PREFLUSH.
  */
 static inline void
 bio_set_flush(struct bio *bio)
 {
-#if defined(HAVE_REQ_PREFLUSH)	/* >= 4.10 */
 	bio_set_op_attrs(bio, 0, REQ_PREFLUSH | REQ_OP_WRITE);
-#elif defined(WRITE_FLUSH_FUA)	/* >= 2.6.37 and <= 4.9 */
-	bio_set_op_attrs(bio, 0, WRITE_FLUSH_FUA);
-#else
-#error	"Allowing the build will cause bio_set_flush requests to be ignored."
-#endif
 }
 
 /*
  * 4.8 API,
  *   REQ_OP_FLUSH
  *
- * 4.8-rc0 - 4.8-rc1,
- *   REQ_PREFLUSH
- *
- * 2.6.36 - 4.7 API,
- *   REQ_FLUSH
- *
  * in all cases but may have a performance impact for some kernels.  It
  * has the advantage of minimizing kernel specific changes in the zvol code.
  *
@@ -468,77 +356,40 @@ bio_set_flush(struct bio *bio)
 static inline boolean_t
 bio_is_flush(struct bio *bio)
 {
-#if defined(HAVE_REQ_OP_FLUSH) && defined(HAVE_BIO_BI_OPF)
-	return ((bio_op(bio) == REQ_OP_FLUSH) || (bio->bi_opf & REQ_PREFLUSH));
-#elif defined(HAVE_REQ_PREFLUSH) && defined(HAVE_BIO_BI_OPF)
-	return (bio->bi_opf & REQ_PREFLUSH);
-#elif defined(HAVE_REQ_PREFLUSH) && !defined(HAVE_BIO_BI_OPF)
-	return (bio->bi_rw & REQ_PREFLUSH);
-#elif defined(HAVE_REQ_FLUSH)
-	return (bio->bi_rw & REQ_FLUSH);
-#else
-#error	"Unsupported kernel"
-#endif
+	return (bio_op(bio) == REQ_OP_FLUSH);
 }
 
 /*
  * 4.8 API,
  *   REQ_FUA flag moved to bio->bi_opf
- *
- * 2.6.x - 4.7 API,
- *   REQ_FUA
  */
 static inline boolean_t
 bio_is_fua(struct bio *bio)
 {
-#if defined(HAVE_BIO_BI_OPF)
 	return (bio->bi_opf & REQ_FUA);
-#elif defined(REQ_FUA)
-	return (bio->bi_rw & REQ_FUA);
-#else
-#error	"Allowing the build will cause fua requests to be ignored."
-#endif
 }
 
 /*
  * 4.8 API,
  *   REQ_OP_DISCARD
  *
- * 2.6.36 - 4.7 API,
- *   REQ_DISCARD
- *
  * In all cases the normal I/O path is used for discards.  The only
  * difference is how the kernel tags individual I/Os as discards.
  */
 static inline boolean_t
 bio_is_discard(struct bio *bio)
 {
-#if defined(HAVE_REQ_OP_DISCARD)
 	return (bio_op(bio) == REQ_OP_DISCARD);
-#elif defined(HAVE_REQ_DISCARD)
-	return (bio->bi_rw & REQ_DISCARD);
-#else
-#error "Unsupported kernel"
-#endif
 }
 
 /*
  * 4.8 API,
  *   REQ_OP_SECURE_ERASE
- *
- * 2.6.36 - 4.7 API,
- *   REQ_SECURE
  */
 static inline boolean_t
 bio_is_secure_erase(struct bio *bio)
 {
-#if defined(HAVE_REQ_OP_SECURE_ERASE)
 	return (bio_op(bio) == REQ_OP_SECURE_ERASE);
-#elif defined(REQ_SECURE)
-	return (bio->bi_rw & REQ_SECURE);
-#else
-	return (0);
-#endif
 }
 
 /*
@@ -579,9 +430,6 @@ bdev_discard_supported(struct block_device *bdev)
  *
  * 4.8 API,
  *   blk_queue_secure_erase()
- *
- * 2.6.36 - 4.7 API,
- *   blk_queue_secdiscard()
  */
 static inline boolean_t
 bdev_secure_discard_supported(struct block_device *bdev)
@@ -590,8 +438,6 @@ bdev_secure_discard_supported(struct block_device *bdev)
 	return (!!bdev_max_secure_erase_sectors(bdev));
 #elif defined(HAVE_BLK_QUEUE_SECURE_ERASE)
 	return (!!blk_queue_secure_erase(bdev_get_queue(bdev)));
-#elif defined(HAVE_BLK_QUEUE_SECDISCARD)
-	return (!!blk_queue_secdiscard(bdev_get_queue(bdev)));
 #else
 #error "Unsupported kernel"
 #endif
@@ -621,10 +467,6 @@ blk_generic_start_io_acct(struct request_queue *q __attribute__((unused)),
 	return (disk_start_io_acct(disk, bio_sectors(bio), bio_op(bio)));
 #elif defined(HAVE_BIO_IO_ACCT)
 	return (bio_start_io_acct(bio));
-#elif defined(HAVE_GENERIC_IO_ACCT_3ARG)
-	unsigned long start_time = jiffies;
-	generic_start_io_acct(rw, bio_sectors(bio), &disk->part0);
-	return (start_time);
 #elif defined(HAVE_GENERIC_IO_ACCT_4ARG)
 	unsigned long start_time = jiffies;
 	generic_start_io_acct(q, rw, bio_sectors(bio), &disk->part0);
@@ -649,8 +491,6 @@ blk_generic_end_io_acct(struct request_queue *q __attribute__((unused)),
 	disk_end_io_acct(disk, bio_op(bio), start_time);
 #elif defined(HAVE_BIO_IO_ACCT)
 	bio_end_io_acct(bio, start_time);
-#elif defined(HAVE_GENERIC_IO_ACCT_3ARG)
-	generic_end_io_acct(rw, &disk->part0, start_time);
 #elif defined(HAVE_GENERIC_IO_ACCT_4ARG)
 	generic_end_io_acct(q, rw, &disk->part0, start_time);
 #endif
@@ -682,7 +522,6 @@ blk_generic_alloc_queue(make_request_fn make_request, int node_id)
 static inline int
 io_data_dir(struct bio *bio, struct request *rq)
 {
-#ifdef HAVE_BLK_MQ
 	if (rq != NULL) {
 		if (op_is_write(req_op(rq))) {
 			return (WRITE);
@@ -690,57 +529,38 @@ io_data_dir(struct bio *bio, struct request *rq)
 			return (READ);
 		}
 	}
-#else
-	ASSERT3P(rq, ==, NULL);
-#endif
 	return (bio_data_dir(bio));
 }
 
 static inline int
 io_is_flush(struct bio *bio, struct request *rq)
 {
-#ifdef HAVE_BLK_MQ
 	if (rq != NULL)
 		return (req_op(rq) == REQ_OP_FLUSH);
-#else
-	ASSERT3P(rq, ==, NULL);
-#endif
 	return (bio_is_flush(bio));
 }
 
 static inline int
 io_is_discard(struct bio *bio, struct request *rq)
 {
-#ifdef HAVE_BLK_MQ
 	if (rq != NULL)
 		return (req_op(rq) == REQ_OP_DISCARD);
-#else
-	ASSERT3P(rq, ==, NULL);
-#endif
 	return (bio_is_discard(bio));
 }
 
 static inline int
 io_is_secure_erase(struct bio *bio, struct request *rq)
 {
-#ifdef HAVE_BLK_MQ
 	if (rq != NULL)
 		return (req_op(rq) == REQ_OP_SECURE_ERASE);
-#else
-	ASSERT3P(rq, ==, NULL);
-#endif
 	return (bio_is_secure_erase(bio));
 }
 
 static inline int
 io_is_fua(struct bio *bio, struct request *rq)
 {
-#ifdef HAVE_BLK_MQ
 	if (rq != NULL)
 		return (rq->cmd_flags & REQ_FUA);
-#else
-	ASSERT3P(rq, ==, NULL);
-#endif
 	return (bio_is_fua(bio));
 }
 
@@ -748,36 +568,24 @@ io_is_fua(struct bio *bio, struct request *rq)
 static inline uint64_t
 io_offset(struct bio *bio, struct request *rq)
 {
-#ifdef HAVE_BLK_MQ
 	if (rq != NULL)
 		return (blk_rq_pos(rq) << 9);
-#else
-	ASSERT3P(rq, ==, NULL);
-#endif
 	return (BIO_BI_SECTOR(bio) << 9);
 }
 
 static inline uint64_t
 io_size(struct bio *bio, struct request *rq)
 {
-#ifdef HAVE_BLK_MQ
 	if (rq != NULL)
 		return (blk_rq_bytes(rq));
-#else
-	ASSERT3P(rq, ==, NULL);
-#endif
 	return (BIO_BI_SIZE(bio));
 }
 
 static inline int
 io_has_data(struct bio *bio, struct request *rq)
 {
-#ifdef HAVE_BLK_MQ
 	if (rq != NULL)
 		return (bio_has_data(rq->bio));
-#else
-	ASSERT3P(rq, ==, NULL);
-#endif
 	return (bio_has_data(bio));
 }
 #endif /* _ZFS_BLKDEV_H */
