@@ -40,4 +40,18 @@
 #define	page_mapping(p) folio_mapping(page_folio(p))
 #endif
 
+/*
+ * 6.12 removed PG_error, SetPageError and ClearPageError, with no direct
+ * replacement, because page writeback errors are recorded elsewhere. Since we
+ * only use the page cache to assist with mmap(), never directly backing it
+ * with IO, it shouldn't be possible for this condition to occur on our pages
+ * anyway, even if this is the right way to report it. So it should be safe
+ * to remove, but for avoidance of doubt, we make it a no-op on 6.12 and leave
+ * it for everything else.
+ */
+#ifndef HAVE_MM_PAGE_FLAG_ERROR
+#define	SetPageError(p)		do {} while (0)
+#define	ClearPageError(p)	do {} while (0)
+#endif
+
 #endif /* _ZFS_MM_COMPAT_H */
