@@ -249,8 +249,7 @@ extern struct proc p0;
 extern kthread_t *zk_thread_create(void (*func)(void *), void *arg,
     size_t stksize, int state);
 
-#define	issig(why)	(FALSE)
-#define	ISSIG(thr, why)	(FALSE)
+#define	issig()		(FALSE)
 
 #define	KPREEMPT_SYNC		(-1)
 
@@ -414,6 +413,7 @@ void procfs_list_add(procfs_list_t *procfs_list, void *p);
 #define	KM_NORMALPRI		0	/* not needed with UMEM_DEFAULT */
 #define	KMC_NODEBUG		UMC_NODEBUG
 #define	KMC_KVMEM		0x0
+#define	KMC_RECLAIMABLE		0x0
 #define	kmem_alloc(_s, _f)	umem_alloc(_s, _f)
 #define	kmem_zalloc(_s, _f)	umem_zalloc(_s, _f)
 #define	kmem_free(_b, _s)	umem_free(_b, _s)
