@@ -249,8 +249,25 @@ zfs_dev_flush(int fd)
 	return (0);
 }
 
+void
+update_vdev_config_dev_sysfs_path(nvlist_t *nv, const char *path,
+    const char *key)
+{
+	(void) nv;
+	(void) path;
+	(void) key;
+}
+
 void
 update_vdevs_config_dev_sysfs_path(nvlist_t *config)
 {
 	(void) config;
 }
+
+int
+zpool_disk_wait(const char *path)
+{
+
+	(void) path;
+	return (ENOTSUP);
+}
