@@ -25,9 +25,6 @@
  *
  */
 
-#include <sys/cdefs.h>
-__FBSDID("$FreeBSD$");
-
 #include <sys/types.h>
 #include <sys/param.h>
 #include <sys/conf.h>
@@ -39,6 +36,7 @@ __FBSDID("$FreeBSD$");
 #include <sys/cmn_err.h>
 #include <sys/zfs_ioctl_compat.h>
 
+#ifdef ZFS_LEGACY_SUPPORT
 enum zfs_ioc_legacy {
 	ZFS_IOC_LEGACY_NONE =	-1,
 	ZFS_IOC_LEGACY_FIRST =	0,
@@ -125,7 +123,7 @@ enum zfs_ioc_legacy {
 	ZFS_IOC_LEGACY_LAST
 };
 
-unsigned static long zfs_ioctl_legacy_to_ozfs_[] = {
+static unsigned long zfs_ioctl_legacy_to_ozfs_[] = {
 	ZFS_IOC_POOL_CREATE,			/* 0x00 */
 	ZFS_IOC_POOL_DESTROY,			/* 0x01 */
 	ZFS_IOC_POOL_IMPORT,			/* 0x02 */
@@ -208,7 +206,7 @@ unsigned static long zfs_ioctl_legacy_to_ozfs_[] = {
 	ZFS_IOC_POOL_INITIALIZE,		/* 0x4d:0x4f */
 };
 
-unsigned static long zfs_ioctl_ozfs_to_legacy_common_[] = {
+static unsigned long zfs_ioctl_ozfs_to_legacy_common_[] = {
 	ZFS_IOC_POOL_CREATE,			/* 0x00 */
 	ZFS_IOC_POOL_DESTROY,			/* 0x01 */
 	ZFS_IOC_POOL_IMPORT,			/* 0x02 */
@@ -297,7 +295,7 @@ unsigned static long zfs_ioctl_ozfs_to_legacy_common_[] = {
 	ZFS_IOC_LEGACY_NONE, /* ZFS_IOC_WAIT_FS */
 };
 
-unsigned static long zfs_ioctl_ozfs_to_legacy_platform_[] = {
+static unsigned long zfs_ioctl_ozfs_to_legacy_platform_[] = {
 	ZFS_IOC_LEGACY_NONE, /* ZFS_IOC_EVENTS_NEXT */
 	ZFS_IOC_LEGACY_NONE, /* ZFS_IOC_EVENTS_CLEAR */
 	ZFS_IOC_LEGACY_NONE, /* ZFS_IOC_EVENTS_SEEK */
@@ -361,3 +359,4 @@ zfs_cmd_ozfs_to_legacy(zfs_cmd_t *src, zfs_cmd_legacy_t *dst)
 	    sizeof (zfs_cmd_t) - 8 - offsetof(zfs_cmd_t, zc_sendobj));
 	dst->zc_jailid = src->zc_zoneid;
 }
+#endif /* ZFS_LEGACY_SUPPORT */
