@@ -25,7 +25,7 @@
 # `configure` is run.
 #
 
-set -e -u
+set -eu
 
 dist=no
 distdir=.
@@ -34,6 +34,7 @@ do
 	case $flag in
 		\?) echo "Usage: $0 [-D distdir] [file]" >&2; exit 1;;
 		D)  dist=yes; distdir=${OPTARG};;
+		*)  ;;
 	esac
 done
 shift $((OPTIND - 1))
@@ -45,6 +46,7 @@ GITREV="${1:-include/zfs_gitrev.h}"
 case "${GITREV}" in
 	/*) echo "Error: ${GITREV} should be a relative path" >&2
 	    exit 1;;
+	*) ;;
 esac
 
 ZFS_GITREV=$({ cd "${top_srcdir}" &&
@@ -60,7 +62,7 @@ then
 			'1s/^#define[[:blank:]]ZFS_META_GITREV "\([^"]*\)"$/\1/p' \
 			"${top_srcdir}/${GITREV}")
 	fi
-elif [ ${dist} = yes ]
+elif [ "${dist}" = yes ]
 then
 	# Append -dist when creating distributed sources from a git repository
 	ZFS_GITREV="${ZFS_GITREV}-dist"
